@@ -81,10 +81,16 @@ extern "C" {
 @private
   NSMutableDictionary   *_nmap;         // Mapping namespaces.
   NSTimeZone	        *_tz;           // Default timezone.
-  BOOL		        _compact;
+  BOOL		        _compact;       // YES for single line output.
   BOOL			_debug;		// YES if debug is enabled.
   BOOL			_fault;		// YES while building a fault.
   BOOL                  _oldparser;     // YES if no namespace support.
+  BOOL                  _crlf;          // YES to use CRLF rather than LF
+  BOOL                  _preferSloppyParser; // Whether the tolerant GNUstep
+                                             // parser should be used.
+  BOOL                  _preserveSpace; // YES to preserve white spece
+  BOOL                  _allUnicode;    // YES to allow all unicode characters
+  BOOL			_cdata;		// YES if we use -characterDataFrom:
   unsigned              _level;         // Current indentation level.
   NSMutableString       *_ms;           // Not retained.
   id                    _delegate;      // Not retained.
@@ -96,6 +102,11 @@ extern "C" {
  */
 + (GWSCoder*) coder;
 
+/**
+ * Return the value set by a prior call to -setCDATA: (or NO ... the default).
+ */
+- (BOOL) cdata;
+
 /**
  * Return the value set by a prior call to -setCompact: (or NO ... the default).
  */
@@ -136,6 +147,11 @@ extern "C" {
  */
 - (NSString*) encodeHexBinaryFrom: (NSData*)source;
 
+/** Takes the supplied string and uses CDATA to escape it for use in an XML
+ * element where character data is allowed.
+ */
+- (NSString*) escapeCDATAFrom: (NSString*)str;
+
 /** Take the supplied string and add all necessary escapes for XML.
  */
 - (NSString*) escapeXMLFrom: (NSString*)str;
@@ -144,13 +160,22 @@ extern "C" {
  */
 - (void) indent;
 
+/** Removes any characters which are not legal in xm version 1.0 leaving
+ * a 'legal' string.  This does not replace special characters such as
+ * ampersand with entities, for that you need to use the -escapeXMLFrom:
+ * method.
+ */
+- (NSString*) legalXMLFrom: (NSString*)str;
+
 /** Returns the mutable string currently in use for encoding.
  */
 - (NSMutableString*) mutableString;
 
 /** Add a new line to the temporary string currently in use for
  * creating an XML document, and add padding on the new line so
- * that the next item written is indented correctly.
+ * that the next item written is indented correctly.<br />
+ * A newline is a linefeed (LF) character unless the -setCRLF
+ * method is used to override that.
  */
 - (void) nl;
 
@@ -174,12 +199,37 @@ extern "C" {
  */
 - (id) parseXSI: (NSString*)type string: (NSString*)value;
 
+/**
+ * Whether characters that are illegal in xml 1.0 should be permitted in our
+ * encoding.  This should be turned on for xml 1.1 use.
+ */
+- (BOOL) permitAllUnicode;
+
+/**
+ * Whether the more tolerant, non-libxml2 parser in GNUstep should be used.
+ */
+- (BOOL) preferSloppyParser;
+
+/**
+ * Whether whitespace in contrent adjacent to element markup should be
+ * stripped/ignored.
+ */
+- (BOOL) preserveSpace;
+
 /**
  * Resets parsing and/or building, releasing any temporary
- * data stored during parse etc.
+ * data stored during parse etc.<br />
+ * This does not alter the effects of the -setCompact:
+ * or -setCRLF: methods.
  */
 - (void) reset;
 
+/** Specifies whether character data content of elements is output using
+ * CDATA sections.  If this is NO, characters are individually escaped
+ * using numeric entities when required.
+ */
+- (void) setCDATA: (BOOL)flag;
+
 /**
  * Specify whether to generate compact XML (omit indentation and other white
  * space and omit &lt;string&gt; element markup for XMLRPC).<br />
@@ -188,10 +238,39 @@ extern "C" {
  */
 - (void) setCompact: (BOOL)flag;
 
-/** Specifies whether debug information is enabled.  See -debug for more
- * information.
+/** Specifies whether newlines between elements are represented as a
+ * single LF or as a CRLF sequence when generating output.<br />
+ * NB this has no effect for compact output (where all output is on a
+ * single line).
  */
-- (void) setDebug: (BOOL)flag;
+- (void) setCRLF: (BOOL)flag;
+
+/** Specifies whether debug information is enabled.  See -debug for more
+ * information.<br />
+ * A non-zero setting sets debg to YES while a zero setting sets it to NO.
+ * The method returns the previous setting.
+ */
+- (int) setDebug: (int)flag;
+
+/**
+ * Specifies whether illegal characters (for xml 1.0) should be permitted
+ * in the output when escaping unicode text.
+ * You should turn this on for xml 1.1 output.
+ */
+- (void) setPermitAllUnicode: (BOOL)flag;
+
+/**
+ * Specifies whether the more tolerant, non-libxml2 parser should be used if
+ * available. This supports a wider range of not-entirely valid XML, but
+ * omits features such as external entity support.
+ */
+- (void) setPreferSloppyParser: (BOOL)flag;
+
+/** Specifies whether leading and trailing whie space character content
+ * of an element is preserved or stripped/ignored (standard xml behavior
+ * is to strip/ignore it).
+ */
+- (void) setPreserveSpace: (BOOL)flag;
 
 /** Decrease the indentation level used while creating an XML document.
  * creating an XML document.
@@ -437,7 +516,7 @@ typedef enum {
 
 /** Take the supplied date and encode it as an XMLRPC timestamp.<br />
  * This uses the timezone currently set in the receiver to determine
- * the time of day encoded.
+ * the time of day encoded.  The format is YYYYMMDDTHH:MM:SS
  */
 - (NSString*) encodeDateTimeFrom: (NSDate*)source;
 
@@ -490,13 +569,18 @@ typedef enum {
  * </p>
  * <p>The JSON coder supports JSON-RPC versions 1.0 and 2.0 as well as
  * supporting simple encoding/decoding of JSON documents without JSON-RPC
- * (where the RPC version is set to nil).
+ * (where the RPC version is set to nil).<br />
+ * To force parsing as JSON-RPC rather than simply a JSON document, call
+ * the -setStrictParsing: method.  In this case, if the jsonrpc field is
+ * missing the request is assumed to be version 1.0 (this field was added
+ * in 2.0) and a value of the jsonrpc field other than 2.0 is illegal. 
  * </p>
  */
 @interface GWSJSONCoder : GWSCoder
 {
   NSString      *_version;      /** Not retained ... default version */
   id            _jsonID;        /** Default request/response ID */
+  BOOL          _jsonrpc;       /** Set to force strict JSON-RPC parsing */
 }
 
 /** This method appends an object to the mutable string in use by the coder.
@@ -544,10 +628,22 @@ typedef enum {
                parameters: (NSDictionary*)parameters
                     order: (NSArray*)order;
 
+/** A helper method which decodes a timestamp from ISO8601 format,
+ * YYYY-MM-DDTHH:MM:SS.mmmZ<br />
+ * This tolerates omission of the trailing Z (in which case it assumes
+ * the local timezone rather than GMT), the millisecond part,
+ * and (in the case that both milliseconds and 'Z' are omitted) also
+ * tolerates omission of the hyphens and colons (YYYYMMDDTHHMMSS).<br />
+ * Returns nil if the argument cannot be decoded.
+ */
+- (NSCalendarDate*) decodeDateTimeFrom: (NSString*)source;
+
 /** Take the supplied date and encode it as a string.<br />
- * This uses the timezone currently set in the receiver to determine
- * the time of day encoded.<br />
- * There is no standard for JSON timestamps.
+ * There is no standard for JSON timestamps, but the recommended value
+ * (supported by javascript) is ISO8601 ... YYYY-MM-DDTHH:MM:SS.mmmZ<br />
+ * If the -setTimeZone: method is used, the format YYYYMMDDTHHMMSS will
+ * be used relative to the time zone value set (unless a nil argument
+ * was supplied).
  */
 - (NSString*) encodeDateTimeFrom: (NSDate*)source;
 
@@ -565,6 +661,17 @@ typedef enum {
  */
 - (void) setRPCID: (id)o;
 
+/** If set, parsing only permits strict JSON-RPC requests/responses.
+ */
+- (void) setStrictParsing: (BOOL)isStrict;
+
+/** Sets a timeZone so that encoding of date/time values will be done
+ * relative to that timeZone as YYYMMDDTHHMMSS format rather than the
+ * standard YYYY-MM-DDTHH:MM:SS.mmZ format using GMT.<br />
+ * Setting a nil timeZone reverts to standard behavior.
+ */
+- (void) setTimeZone: (NSTimeZone*)timeZone;
+
 /** Sets the json-rpc version.<br />
  * May be "2.0" or "1.0" (any other non-nil value is currently considered
  * to be version 1.0) or nil if JSON-RPC is not to be used.<br />
@@ -575,6 +682,10 @@ typedef enum {
  */
 - (void) setVersion: (NSString*)v;
 
+/** Returns whether strict parsing as JSON-RPC has been configured.
+ */
+- (BOOL) strictParsing;
+
 /** Returns the json-rpc version (currently "2.0" or "1.0" or nil).<br />
  * See -setVersion: for details.
  */
