@@ -93,11 +93,11 @@ extern "C" {
 - (GWSElement*) addChildNamed: (NSString*)name
 		    namespace: (NSString*)namespace
 		    qualified: (NSString*)qualified
-		      content: (NSString*)content, ...;
+		      content: (NSString*)content,
+			   ... NS_REQUIRES_NIL_TERMINATION;
 		    
 /** Adds a string to the content of the receiver.  New content is appended
- * to any existing content (with any white space between the two parts
- * being condensed to a single space).
+ * to any existing content.
  */
 - (void) addContent: (NSString*)content;
 
@@ -106,8 +106,8 @@ extern "C" {
  */
 - (NSString*) attributeForName: (NSString*)name;
 
-/** Returns the attributes of the receiver, or an empty dictionary
- * if no attributes are set.
+/** Returns an autoreleased immutable copy of the attributes of the receiver,
+ * or an empty dictionary if no attributes are set.
  */
 - (NSDictionary*) attributes;
 
@@ -121,9 +121,15 @@ extern "C" {
  */
 - (NSArray*) children;
 
+/** Condenses the character content of the receiver by stripping leading
+ * and trailing white space.  If the internal flag is YES, any sequence of
+ * internal white space is also condensed to a single character by deleting
+ * all but the first character.
+ */
+- (void) condense: (BOOL)internal;
+
 /** Returns the content text of the receiver.  This may be an empty string
- * if no content has been added to the receiver (but will never be nil).<br />
- * The returned value will have no leading or trailing white space.
+ * if no content has been added to the receiver (but will never be nil).
  */
 - (NSString*) content;
 
@@ -215,7 +221,10 @@ extern "C" {
 
 /* <init />
  * Initialises the receiver with the name, namespace URI, fully qualified
- * name, and attributes given.
+ * name, and attributes given.<br />
+ * The receiver makes a mutable copy of the attributes dictionary, and may
+ * subsequently modify that copy in response to the -setAttribute:forKey:
+ * method.
  */
 - (id) initWithName: (NSString*)name
           namespace: (NSString*)namespace
@@ -330,7 +339,7 @@ extern "C" {
 - (void) setAttribute: (NSString*)attribute forKey: (NSString*)key;
 
 /** Sets the value of the content string of the receiver, replacing any
- * content already present.  Any leading white space is removed.<br />
+ * content already present.<br />
  * You may use an empty string or nil to remove content from the
  * receiver.
  */
