@@ -27,6 +27,7 @@
 #define	INCLUDED_GWSSERVICE_H
 
 #import <Foundation/NSObject.h>
+#import <Foundation/NSDebug.h>
 
 #if     defined(__cplusplus)
 extern "C" {
@@ -91,6 +92,9 @@ extern "C" {
  * </p>
  */
 @interface	GWSService : NSObject
+#if	defined(GNUSTEP)
+  <GSLogDelegate>
+#endif
 {
 @private
   NSString              *_name;
@@ -106,12 +110,14 @@ extern "C" {
   id			_delegate;	// Not retained.
   NSTimeZone		*_tz;
   GWSCoder              *_coder;
+  NSString		*_HTTPMethod;
   NSString		*_SOAPAction;
   BOOL			_compact;
   BOOL			_debug;
   BOOL			_prioritised;
   BOOL			_cancelled;	// Timeout occurred
   BOOL			_completedIO;	// Comms completed
+  BOOL			_newAPI;
   NSString		*_operation;
   GWSPort		*_port;
   NSMutableDictionary	*_parameters;
@@ -143,6 +149,11 @@ extern "C" {
  */
 + (NSString*) description;
 
+/** Empty the cache of connections being kept alive to the specified
+ * URL or, if url is nil empty the entire cache.
+ */
++ (void) flushConnections: (NSURL*)url;
+
 /** Sets maximum active requests to a single host.  This is silently limited
  * to be no more than the value set by the +setPool: method.
  */
@@ -235,6 +246,18 @@ extern "C" {
  */
 - (GWSElement*) documentation;
 
+/** Method called (if GNUstep debug logging is turned on) when a chunk of
+ * data from the HTTP request has been written.  The default implementation
+ * returns NO (so default logging is used) unless the delegate supports the
+ * method (in which case the delegate method is used), but subclasses may
+ * override it to provide their own logging and return YES.
+ * When the delegate method is called, the handle sent to it is the GWSService
+ * instance. 
+ */
+- (BOOL) getBytes: (const uint8_t*)bytes
+         ofLength: (NSUInteger)length
+         byHandle: (NSObject*)anObject;
+
 /** Returns the value set by a previous call to the -setHeaders: method.
  */
 - (NSDictionary*) headers;
@@ -263,6 +286,18 @@ extern "C" {
  */
 - (id) objectForKey: (NSString*)aKey;
 
+
+/** Method called (if GNUstep debug logging is turned on) when a chunk of
+ * data from the HTTP request has been written.  The default implementation
+ * returns NO (so default logging is used) unless the delegate supports the
+ * method (in which case the delegate method is used), but subclasses may
+ * override it to provide their own logging and return YES.
+ * When the delegate method is called, the handle sent to it is the GWSService
+ * instance. 
+ */
+- (BOOL) putBytes: (const uint8_t*)bytes
+         ofLength: (NSUInteger)length
+         byHandle: (NSObject*)anObject;
 /**
  * Returns the result of the last method call, or nil if there has been
  * no method call or one is in progress.<br />
@@ -325,9 +360,11 @@ extern "C" {
 - (void) setContentType: (NSString*)cType;
 
 /** Specifies whether debug information is enabled.  See -debug for more
- * information.
+ * information.<br />
+ * A non-zero setting sets debug to YES while a zero setting sets it to NO.
+ * The method returns the previous setting.
  */
-- (void) setDebug: (BOOL)flag;
+- (int) setDebug: (int)flag;
 
 /**
  * Sets the delegate object which will receive callbacks when an RPC
@@ -350,6 +387,20 @@ extern "C" {
  */
 - (void) setHeaders: (NSDictionary*)headers;
 
+/** Sets the value of the HTTP method to be used for a request.<br />
+ * Setting an nil string reverts to the default 'POST' method.<br />
+ */
+- (void) setHTTPMethod: (NSString*)method;
+
+/** Controls whether the new (NSURLConnection/NSURLRequest) API is used
+ * or the old (NSURLHandle) API is used.  The default depends on the
+ * platform (GNUstep uses the old API while Apple uses the new).<br />
+ * Turning on debug or the use of client certificates overrides this
+ * to use the old API since the new one does not yet support those
+ * features.
+ */
+- (BOOL) setNewAPI: (BOOL)aFlag;
+
 /** Tags the service with entra information in the form of anObject
  * references by aKey.  if anObject is nil, this removes any previous
  * association for aKey.<br />
@@ -359,8 +410,7 @@ extern "C" {
  */
 - (void) setObject: (id)anObject forKey: (NSString*)aKey;
 
-/**
- * Sets the value of the SOAPAction header to be sent with a request.<br />
+/** Sets the value of the SOAPAction header to be sent with a request.<br />
  * Setting an nil string value suppresses the sending of this header.<br />
  * Most servers expect two double quotes characters for this,
  * so you should probably set @&quot;\&quot;\&quot;&quot; as the action.<br />
@@ -441,7 +491,6 @@ extern "C" {
  * performed.
  */
 - (GWSPort*) webServicePort;
-
 @end
 
 /**
@@ -540,11 +589,8 @@ extern "C" {
  * (such as the URL to send to) before the data is actualy sent.
  */
 - (NSData*) webService: (GWSService*)sender willSendRequest: (NSData*)data;
-
 @end
 
-
-
 #if	defined(__cplusplus)
 }
 #endif
